@@ -1,7 +1,7 @@
 #
 # RT was configured with:
 #
-#   $ ./configure --with-rt-group=bin --enable-layout=OpenBSD --enable-gpg --enable-graphviz --enable-gd --prefix=/usr/local --sysconfdir=/etc --mandir=/usr/local/man --infodir=/usr/local/info --localstatedir=/var --disable-silent-rules
+#   $ ./configure --with-rt-group=bin --enable-layout=OpenBSD --enable-gpg --enable-graphviz --enable-gd --prefix=/usr/local --sysconfdir=/etc --mandir=/usr/local/man --infodir=/usr/local/info --localstatedir=/var --disable-silent-rules --disable-gtk-doc
 #
 
 package RT;
@@ -121,7 +121,7 @@ installed).
 
 Example:
 
-C<Set(@Plugins, (qw(Extension::QuickDelete RT::Extension::CommandByMail)));>
+C<Set(@Plugins, (qw(RT::Extension::SLA RT::Authen::ExternalAuth)));>
 
 =cut
 
